!*****************************************************************************
Program TestDiagonalisation
!****************************************************************************

  Use ConjGrad
  Use TypeDef, only : dbl

  Implicit None

  integer :: HamSize,NEig,r
  real(dbl) :: tol
  real(dbl), dimension(:), allocatable :: EigValues
  complex(dbl), dimension(:,:), allocatable :: EigStates, HMatrix

  HamSize = 180
  NEig = 10
  tol = 1.d-8

  allocate(HMatrix(HamSize,HamSize),EigValues(NEig), &
       & EigStates(HamSize,NEig))

  print *, 'N: ',HamSize,'NEig: ',NEig,'tol:',tol

  Call FillHMatrix(HMatrix,HamSize)

  Call CGDiagonalise(HamSize,HMatrix,NEig,EigValues,EigStates,tol)



  deallocate(HMatrix,EigValues,EigStates)

Contains
!***************************************************************************
  Subroutine FillHMatrix(HMatrix,HamSize)
!***************************************************************************

    integer, intent(in) :: HamSize
    complex(dbl), dimension(:,:) :: HMatrix

    integer :: r,s
    real(dbl) :: offdiag

    do r = 1, HamSize
       do s = r, HamSize
          if(r.eq.s) then
             HMatrix(r,r) = cmplx(dble(r)**(2.d0/3.d0),0.d0)
          else
             offdiag = sqrt(dble(r+s))-int(sqrt(dble(r+s)))-0.5d0 
             HMatrix(r,s) = cmplx(offdiag,offdiag)
             HMatrix(s,r) = conjg(HMatrix(r,s))
          end if
       end do
    end do

  end Subroutine FillHMatrix

!****************************************************************************

end Program TestDiagonalisation

!****************************************************************************
