!!****************************************************************************
Module Minimisation
!****************************************************************************

  Use TypeDef
  Use Normalisation,  only : SelfBandOrthogonalise, VecNormalise

  Implicit None

Contains

!****************************************************************************
  Subroutine StDescentVec(EigSystem,MinVec,iband)
!****************************************************************************

    integer, intent(in) :: iband
    type(EigSystemT), pointer :: EigSystem
    type(MinVecT), pointer :: MinVec

    ! Evaluate the Steepest Descent direction for iband
    MinVec%StDesc = - EigSystem%EPsi + EigSystem%Values(iband)* &
         & EigSystem%States(:,iband)

!    print *, 'StDesc:',MinVec%StDesc

  end Subroutine StDescentVec

!****************************************************************************
  Subroutine ConjDirection(EigSystem,MinVec,OldVec,iband)
!****************************************************************************

    integer, intent(in) :: iband
    type(EigSystemT), pointer :: EigSystem
    type(MinVecT), pointer :: MinVec,OldVec

    real(dbl) :: Gamma,numer,denom

    !   Calculate Gamma, the phi is adjusted such that in the first 
    ! iteration Gamma becomes inconsequential
    numer = Dot_Product((MinVec%StDesc-OldVec%StDesc), MinVec%StDesc)
    denom = Dot_Product((MinVec%StDesc-OldVec%StDesc), OldVec%Phi)

    Gamma = -numer/(denom + 10.d-40)

!    Gamma = 0.d0     ! This makes it the steepest descent method

    MinVec%Phi = MinVec%StDesc + Gamma*OldVec%Phi

    ! Orthogonalise Phi against the state-vector of the same band 'iband'
    Call SelfBandOrthogonalise(EigSystem,MinVec%Phi,iband)

    ! Normalise Phi
    Call VecNormalise(MinVec%Phi)

!    print *, 'Conjgdir:',MinVec%Phi

    ! Roll the current values into OldVec
    OldVec%Phi = MinVec%Phi
    OldVec%StDesc = MinVec%StDesc

  end Subroutine ConjDirection

!*****************************************************************************

end Module Minimisation


!*****************************************************************************
