!*****************************************************************************
Module Normalisation
!*****************************************************************************

  Use TypeDef

  Implicit None

Contains
!***************************************************************************
  Subroutine EigStateOrth(EigSystem,iband)
!****************************************************************************
!
!   This Subroutine orthonormalises the starting vector for iband against
! all the lower bands.
!
    integer, intent(in) :: iband
    type(EigSystemT), pointer :: EigSystem

    ! Orthogonalise to all the LOWER bands
    Call LowerBandOrthogonalise(EigSystem,EigSystem%States(:,iband),iband)

    ! Normalise the state-vector
    Call VecNormalise(EigSystem%States(:,iband))

!    print *, 'OrthStartVec:',EigSystem%States(:,iband)

  end Subroutine EigStateOrth
!*****************************************************************************
  Subroutine LowerBandOrthogonalise(EigSystem,Vector,iband)
!*****************************************************************************
!
!   This Subroutine Orthogonalises the Vector corresponding to iband against 
! all the bands (in EigSystem) below iband.

    integer, intent(in) :: iband
    type(EigSystemT), pointer :: EigSystem
    complex(dbl), dimension(:) :: Vector
    
    integer :: r,jband, NSize
    complex(dbl) :: orthfac

    NSize = EigSystem%HSize

    do jband = 1, iband-1

       !    Calculate the orthogonalising factor for the j th band wrt 
       ! the i th band.
       orthfac = Dot_Product(EigSystem%States(:,jband),Vector)

       !   Calculate the Orthogonalising Vector between i and j bands
       Vector = Vector - orthfac*EigSystem%States(:,jband)
          
    end do

  end Subroutine LowerBandOrthogonalise

!*****************************************************************************
  Subroutine VecNormalise(Vector)
!****************************************************************************
!
!   This Subroutine normalises Vector

    complex(dbl), dimension(:) :: Vector
    real(dbl) :: norm

    norm = Dot_Product(Vector,Vector)
    norm = sqrt(norm) + 1.d-32
!    if(norm.le.1.d-15) then
!       Vector = cmplx(0.d0,0.d0)
!    else
       Vector = Vector/norm
!    end if

  end Subroutine VecNormalise

!***************************************************************************
  Subroutine SelfBandOrthogonalise(EigSystem,Vector,iband)
!*****************************************************************************
!
!   This Subroutine orthogonalises the Vector corresponding to iband, against
! the state-vector iband contained in EigSystem.
!

    integer, intent(in) :: iband
    type(EigSystemT), pointer :: EigSystem
    complex(dbl), dimension(:) :: Vector
    complex(dbl) :: orthfac

    orthfac = Dot_Product(EigSystem%States(:,iband),Vector)

    Vector = Vector - orthfac*EigSystem%States(:,iband)
       

  end Subroutine SelfBandOrthogonalise

!****************************************************************************
  Subroutine BandOrthogonalise(EigSystem,Vector,iband)
!*****************************************************************************
!
!   This Subroutine orthogonalises the Vector corresponding to iband against
! all the bands below and including iband.
! 
    integer, intent(in) :: iband
    type(EigSystemt), pointer :: EigSystem
    complex(dbl), dimension(:) :: Vector

    ! First Orthogonalise against iband
    Call SelfBandOrthogonalise(EigSystem,Vector,iband)

    ! Next Orthogonalise that against all the lower bands
    Call LowerBandOrthogonalise(EigSystem,Vector,iband)


  end Subroutine BandOrthogonalise

!*****************************************************************************




end Module Normalisation

!*****************************************************************************
